!w =================================================================================================================================
! MODULE       : erosion
!
! CONTACT      : Haicheng Zhang (sysuzhaicheng@163.com), Ronny Lauerwald
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       This module calculating the daily water-erosion rate.
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S) :
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================
!!=========================================================================================
!!! Latest changes ------- by ZHC
!!! Time = 20170809
!!! Change contents: 
!!!   1) The eroded soil carbon and belowground litter is calculated based on the carbon and litter contents in the top 7 soil layers,
!!!   rather than based on the carbon and litter contents in the top 8 soil layers in previous version.
!!=========================================================================================
MODULE erosion
  !
  !
  ! routines called : restput, restget
  !
  USE time, ONLY : one_day, dt_sechiba
  USE ioipsl   
  USE xios_orchidee
  USE ioipsl_para 
  USE constantes
  USE constantes_soil
  USE constantes_var
  USE vertical_soil_var
  USE pft_parameters
  USE sechiba_io_p
  USE slowproc
  USE interpol_help
  USE grid
  USE mod_orchidee_para

  IMPLICIT NONE
 
  ! public routines :
 
  PRIVATE
  PUBLIC :: erosion_initialize, erosion_main, erosion_clear
 
  ! variables shared by all subroutines inside the erosion module : declaration and initialisation 
  LOGICAL, SAVE                                   :: l_first_erosion=.TRUE.      !! Logical to initialize the routine.
                                                                                 !! Initialisation has to be done one 
										    								     !! time (true/false)
  REAL(r_std),SAVE                                :: dt_erosion                  !! Erosion time step (s)
  REAL(r_std),SAVE                                :: time_counter_ero            !! Time counter (s)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)  :: gridarea                    !! grid area (m^2)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)  :: cveg_f                      !! vegetation cover factor, unitless, 0-1
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)  :: kflitter                    !! vegetation cover factor, unitless, 0-1
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)  :: kfroot                    !! vegetation cover factor, unitless, 0-1
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)    :: meanrunoff                  !! Accumulated runoff(kg/m^{-2} = mm) 
                                                                                 !! @tex $(kg m^{-2} dt^{-1})$ @endtex		
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)    :: peakrunoff                  !! Peak runoff over one day (mm/s) 
                                                                                 !! @tex $(kg m^{-2} dt^{-1})$ @endtex
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)    :: meanrundra                  !! Accumulated runoff+drainage(kg/m^{-2} = mm) 
                                                                                 !! @tex $(kg m^{-2} dt^{-1})$ @endtex		
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)    :: peakrundra                  !! Peak (runoff +drainage) over one day (mm/s) 
                                                                                 !! @tex $(kg m^{-2} dt^{-1})$ @endtex															 
  INTEGER(r_std), SAVE, ALLOCATABLE, DIMENSION(:) 		:: no_events             !! number of rainfall events
                                                                                 !! @tex $(kg m^{-2} dt^{-1})$ @endtex	
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)        :: eroc_active         	 !! Active SOC export (gC/m^2/day)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)        :: eroc_slow         	 !! Slow SOC export (gC/m^2/day)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)        :: eroc_passive          !! Passive SOC export (gC/m^2/day)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:,:)      :: tot_7c                !! Total Carbon concentration in the top 7 soil layers  (gC m^{-2})
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:,:)      :: tot_7litbelow         !! Total belowground litter in the top 7 soil layers (gC m^{-2})
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:,:,:)    :: tot_7doc              !! Total DOC in the top 7 soil layers (gC m^{-2})
  !!********************* End veraibles for history files ***********************
CONTAINS
!! ======================================================================================================================
!! SUBROUTINE   : erosion_initialize
!! Initialization of the erosion module
!! Read the reference soil erosion rate
!! ======================================================================================================================
!! 1. Initialize
  SUBROUTINE erosion_initialize(kjit,nbpt,index,rest_id, lalo,neighbours,resolution,contfrac,std_totsed,gridarea,effgrid_perc)
	!!
	IMPLICIT NONE
!	LOGICAL, SAVE                                   :: l_first_erosion=.TRUE.      !! Logical to initialize the routine.
    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)     :: kjit                 !! Time step number (unitless)
    INTEGER(i_std), INTENT(in)     :: nbpt                 !! Domain size (unitless)
    INTEGER(i_std), INTENT(in)     :: index(nbpt)          !! Indices of the points on the map (unitless)
    INTEGER(i_std),INTENT(in)      :: rest_id              !! Restart file identifier (unitless)
	REAL(r_std), INTENT(in)        :: lalo(nbpt,2)              !! Vector of latitude and longitudes (beware of the order !)
    INTEGER(i_std), INTENT(in)     :: neighbours(nbpt,8)        !! Vector of neighbours for each grid point 
                                                                !! (unitless; 1=N, 2=NE, 3=E, 4=SE, 5=S, 6=SW, 7=W, 8=NW)
    REAL(r_std), INTENT(in)        :: resolution(nbpt,2)        !! The size of each grid box in X and Y (m)
	REAL(r_std), INTENT(in)        :: contfrac(nbpt)            !! Fraction of land in each grid box (unitless;0-1)
	!! Output variables
	REAL(r_std), INTENT(out)       :: std_totsed(nbpt)        !! standard sediment discharge amount rate for each ORCHIDEE cell(ton)
	REAL(r_std), INTENT(out)       :: gridarea(nbpt)          !! Area of each grid-cell ()
	REAL(r_std), INTENT(out)       :: effgrid_perc(nbpt)      !! percentage of the effective area in a ORCHIDEE pixel (0-1)
	!! Local variables
	INTEGER(i_std)                 :: ig
    !! 1.1 First call only
	dt_erosion = one_day
	WRITE(numout,*) 'Start the first time calling of erosion module', l_first_erosion	
    IF (l_first_erosion) THEN
		WRITE(numout,*) 'This is the first time calling of erosion module'	
		ALLOCATE(cveg_f(nbpt,nvm))
		cveg_f = zero
		ALLOCATE(kflitter(nbpt,nvm))
		kflitter = zero
		ALLOCATE(kfroot(nbpt,nvm))
		kfroot = zero
		ALLOCATE(peakrunoff(nbpt))
		peakrunoff = zero		
		ALLOCATE(meanrunoff(nbpt))
		meanrunoff = zero
		ALLOCATE(peakrundra(nbpt))
		peakrundra = zero		
		ALLOCATE(meanrundra(nbpt))
		meanrundra = zero
		ALLOCATE(eroc_active(nbpt,nvm))
		eroc_active = zero
		ALLOCATE(eroc_slow(nbpt,nvm))
		eroc_slow = zero
		ALLOCATE(eroc_passive(nbpt,nvm))
		eroc_passive = zero
		ALLOCATE(tot_7c(nbpt,ncarb,nvm))
		tot_7c = zero
		ALLOCATE(tot_7litbelow(nbpt,nlitt,nvm))
		tot_7litbelow = zero
		ALLOCATE(tot_7doc(nbpt,nvm,ndoc,npool))
		tot_7doc = zero
		ALLOCATE(no_events(nbpt))		
		no_events = 0
		! Reading input data for calculating erosion (flacc, fldir, std_totsed)
		WRITE(numout,*) 'Start read erosion map'	
		CALL erosionmap(nbpt,index,lalo,neighbours,resolution,contfrac,std_totsed,gridarea,effgrid_perc)
		! Test reading input data
		!DO ig=1,nbpt
		!   WRITE(numout,*) 'igrid', ig,'lat',lalo(ig,1),'lon',lalo(ig,2)
		!ENDDO
!        WRITE(numout,*) 'std_totsed', std_totsed(:)
!		WRITE(numout,*) 'gridarea', gridarea(:)
!		WRITE(numout,*) 'effgrid_perc', effgrid_perc(:)
		!
		l_first_erosion = .FALSE.
		WRITE(numout,*) 'End first_erosion'
	ENDIF !l_first_erosion
  END SUBROUTINE erosion_initialize
!! ======================================================================================================================================
!! SUBROUTINE   : erosion_main
!! ======================================================================================================================================																							
  SUBROUTINE erosion_main(kjit,nbpt,rest_id,dt_sechiba,index, ldrestart_read, ldrestart_write, &
       & lalo,neighbours,resolution,temp_sol,bulkdens,textfrac,laydep,contfrac,runoff,drainage,veget, &
	   & veget_max,hist_id,std_totsed,gridarea,effgrid_perc,rootmass,litter_above,litter_below, &
	   & carbon, DOC, lignin_struc_above,lignin_struc_below,depth_deepsoil,seddep_rate, &
	   & sed_deposition_d, poc_deposition_d,totrunoff_d,peakrunoff_d,totrundra_d,peakrundra_d, &
	   & cf_veget,kf_litter,kf_root,erocap,erodep,eroc,tot_erocap,ave_erodep,tot_eroc, &
	   & POC_EXP_agg,DOC_ERO_agg,SED_EXP_agg)
    IMPLICIT NONE
	!WRITE(numout,*) 'Erosion simulation0', fldir(1238726)
    !! 0.1 Input variables
	INTEGER(i_std), INTENT(in)     :: kjit                      !! Time step number (unitless)
    INTEGER(i_std), INTENT(in)     :: nbpt                      !! Domain size  (unitless)
    INTEGER(i_std),INTENT(in)      :: rest_id                   !! Restart file identifier (unitless)
	REAL(r_std), INTENT(in)        :: runoff(nbpt)         		!! Grid-point runoff @tex $(kg m^{-2} dt^{-1}, mm/dt)$ @endtex	
	REAL(r_std), INTENT(in)        :: drainage(nbpt)         	!! Grid-point drainage @tex $(kg m^{-2} dt^{-1}, mm/dt)$ @endtex	
    REAL(r_std), INTENT(in)        :: dt_sechiba        		!! Time step of the model (s)	
    INTEGER(i_std), INTENT(in)     :: index(nbpt)               !! Indices of the points on the map (unitless)
	REAL(r_std), INTENT(in)        :: lalo(nbpt,2)              !! Vector of latitude and longitudes (beware of the order !)
    INTEGER(i_std), INTENT(in)     :: neighbours(nbpt,8)        !! Vector of neighbours for each grid point 
                                                                !! (unitless; 1=N, 2=NE, 3=E, 4=SE, 5=S, 6=SW, 7=W, 8=NW)
    REAL(r_std), INTENT(in)        :: resolution(nbpt,2)        !! The size of each grid box in X and Y (m)
	REAL(r_std), INTENT(in)        :: contfrac(nbpt)            !! Fraction of land in each grid box (unitless;0-1)
	INTEGER(i_std),INTENT(in)      :: hist_id                   !! Access to history file (unitless)
	REAL(r_std),INTENT (in)        :: veget(nbpt,nvm)           !! Fraction of vegetation type (unitless;0-1)
    REAL(r_std), INTENT(in)        :: veget_max(nbpt,nvm)       !! Maximal fraction of vegetation (unitless;0-1)	
    LOGICAL, INTENT(in)            :: ldrestart_read       		!! Logical for _restart_ file to read (true/false)
    LOGICAL, INTENT(in)            :: ldrestart_write      		!! Logical for _restart_ file to write (true/false)
    REAL(r_std), INTENT(in)        :: temp_sol(nbpt)            !! Surface temperature (K)	
	REAL(r_std), INTENT (in)       :: bulkdens(nbpt)            !! bulk_density (kg m-3)    zhc
	REAL(r_std), INTENT (in)       :: textfrac(nbpt,nctext)     !! Fraction of different soil particle sizes (0-1)
	REAL(r_std), INTENT(in)        :: std_totsed(nbpt)          !! standard sediment discharge amount rate for each ORCHIDEE cell(ton)
	REAL(r_std), INTENT(in)        :: effgrid_perc(nbpt)        !! percentage of the effective area in a ORCHIDEE pixel (0-1)
	REAL(r_std), INTENT(in)        :: gridarea(nbpt)            !! area of each grid cell (m^2)
	!!
    !!0.2 modified variables
	REAL(r_std), DIMENSION (nbpt,nvm,nparts,nelements), INTENT (inout)          :: rootmass      		!! belowground biomass
																										!! @tex $(gC m^{-2})$ @endtex !!ORCHIDEE grid
	REAL(r_std), DIMENSION (nbpt,nlitt,nvm,nelements), INTENT (inout)           :: litter_above  		!! Above ground metabolic and structural litter 
																										!! @tex $(gC m^{-2})$ @endtex !!ORCHIDEE grid
    REAL(r_std), DIMENSION (nbpt,nlitt,nvm,nslmd,nelements), INTENT (inout)     :: litter_below   		!! Below ground metabolic and structural litter 
																										!! per ground area @tex $(gC m^{-2})$ !!ORCHIDEE grid
	REAL(r_std), DIMENSION (nbpt,ncarb,nvm,nslmd), INTENT (inout)               :: carbon        		!! Soil carbon pools per ground area: active, slow, or 
																										!! passive, @tex $(gC m^{-2})$ @endtex !!ORCHIDEE grid
	REAL(r_std), DIMENSION(nbpt,nvm,nslmd,ndoc,npool,nelements), INTENT(inout)  :: DOC           		!! Dissolved Organic Carbon in soil
																										!! The unit is given by m^2 of
																										!! ground @tex $(gC m{-2} of ground)$ @endtex
	REAL(r_std), DIMENSION(nbpt,nvm), INTENT(inout)         					:: lignin_struc_above   !! Ratio Lignin content in structural litter,
																										!! above ground, (0-1, unitless)
    REAL(r_std), DIMENSION(nbpt,nvm,nslmd), INTENT(inout)  						:: lignin_struc_below   !! Ratio Lignin content in structural litter,
																										!! below ground, (0-1, unitless)
	REAL(r_std), DIMENSION(nbpt,nctext), INTENT(inout)        					:: sed_deposition_d   	!! Daily deposition of sediment at floodplains
																										!! @tex $(kg m^2 day^{-1})$ @endtex	
	REAL(r_std), DIMENSION(nbpt,ncarb), INTENT(inout)        					:: poc_deposition_d		!! Daily deposition of POC at floodplains
																										!! @tex $(kg m^2 day^{-1})$ @endtex	
	REAL(r_std), DIMENSION(nbpt,nvm), INTENT (inout)        					:: depth_deepsoil       !! Depth of the soil layer deeper than 2 m (m)
	!!
	!! 0.3 output variables
	REAL(r_std), DIMENSION(nslm),INTENT (out)           :: laydep                   !! The lower limit of each (11) soil layer (m) ::zhc
	REAL(r_std), DIMENSION(nbpt,nvm),INTENT (out)       :: seddep_rate              !! Sediment deposition rate (m day-1) for each PFT (m/day)
	REAL(r_std), DIMENSION(nbpt),INTENT(out)   		    :: totrunoff_d              !! daily total runoff (mm)
	REAL(r_std), DIMENSION(nbpt),INTENT(out)   		    :: peakrunoff_d             !! daily peak runoff rate.(mm/s = kg/m-2/s)
	REAL(r_std), DIMENSION(nbpt),INTENT(out)   		    :: totrundra_d              !! daily total runoff+drainage (mm)
	REAL(r_std), DIMENSION(nbpt),INTENT(out)   		    :: peakrundra_d             !! daily peak (runoff+drainage) rate.(mm/s = kg/m-2/s)
	REAL(r_std), DIMENSION(nbpt,nvm),INTENT(out)   		:: erocap                   !! Erosion capacity from subgrid.(ton)
	REAL(r_std), DIMENSION(nbpt,nvm),INTENT(out)   		:: erodep                   !! Erosion depth of sub_grid (m) under different PFTs (m)
	REAL(r_std), DIMENSION(nbpt,ncarb,nvm),INTENT(out)  :: eroc                     !! Carbon export (gC/m^2/day)
	REAL(r_std), DIMENSION(nbpt),INTENT(out)   		    :: tot_erocap               !! Erosion capacity from subgrid.(ton)
	REAL(r_std), DIMENSION(nbpt),INTENT(out)   		    :: ave_erodep               !! Erosion depth of sub_grid (m)
	REAL(r_std), DIMENSION(nbpt,ncarb),INTENT(out)      :: tot_eroc                 !! Carbon export (gC/m^2/day)
	REAL(r_std), DIMENSION(nbpt,nvm),INTENT(out)   		:: cf_veget                 !! Daily vegetation cover factor
	REAL(r_std), DIMENSION(nbpt,nvm),INTENT(out)   		:: kf_litter                !! Daily aboveground litter cover factor
	REAL(r_std), DIMENSION(nbpt,nvm),INTENT(out)   		:: kf_root                  !! Daily belowground litter cover factor
	REAL(r_std), DIMENSION(nbpt,ncarb),INTENT(out) 	    :: POC_EXP_agg  		    !! POC exports along with surface runoff, in  
																					!! @tex $(gC m^{-2} day^{-1})$ @endtex 
	REAL(r_std), DIMENSION(nbpt,nctext),INTENT(out) 	:: SED_EXP_agg  		    !! sediment exports along with surface runoff, in  
																					!! @tex $(g m^{-2} day^{-1})$ @endtex
	REAL(r_std), DIMENSION(nbpt,ncarb),INTENT(out)      :: DOC_ERO_agg  		    !! DOC exports along with surface runoff, in  
																					!! @tex $(g m^{-2} day^{-1})$ @endtex																				
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!
	!! 0.4 Local variables					
    INTEGER(i_std)             :: ig,jg,lg,fg,ng,ii,icarb,ilitt,ibdl !! integers for loops
	INTEGER(i_std)             :: ilat,ilon,inbpt		             !! temperal variable for latitude and longitude
	INTEGER(i_std)             :: max_flacc		             	  !! maximum value of flow accumulation
	REAL(r_std)                :: erocap_jg                       !! temperal variable for erosion capacity
	REAL(r_std)                :: meanrun_up,peakrun_up,meanrundra_up,peakrundra_up,cveg_up,kflitter_up,kfroot_up !! temperal variable for erosion capacity
	REAL(r_std)                :: littotal,belowbiom              !! vegetation cover fraction (0-1)
	REAL(r_std)                :: vegf                 			  !! vegetation cover fraction (0-1)
	REAL(r_std)                :: max_depth                 	  !! vegetation cover fraction (0-1)
	REAL(r_std)                :: lignin_ave                      !! Average ratio of Lignin content in belowground structural litter in top 7 soil layers
!	REAL(r_std)                :: ave_c7,ave_lit                  !! temperal variable for erosion capacity
!	REAL(r_std)                :: nsubcell			        	  !! potentially maximum number of finer cells(erosion map)
    CHARACTER(LEN=80)          :: var_name      	              !! To store variables names for I/O (unitless)	
	CHARACTER(LEN=10)          :: carb_suff                       !! To assign a suffix to the variables name indicating carbon type
	                                                              !! (active, slow, passive)
	CHARACTER(LEN=10)          :: litt_suff                       !! To assign a suffix to the variables name indicating litterfall type
	                                                              !! (structual, metabolic)
    CHARACTER(LEN=10)          :: solly_suff                      !! To store variables names for I/O soil layer																
	REAL(r_std), DIMENSION(nbpt,ncarb,nvm,nslmd) ::carbon_old    !! Soil carbon pools per ground area: active, slow, or 
                                                                  !! passive, @tex $(gC m^{-2})$ @endtex !!ORCHIDEE grid
	REAL(r_std), DIMENSION(nbpt,nlitt,nvm,nslmd) ::litbe_old     !! belowground litter pools per ground area: 
                                                                  !! @tex $(gC m^{-2})$ @endtex !!ORCHIDEE grid
	REAL(r_std), DIMENSION(nbpt,nvm,nslmd) ::litbe_tot           !! total belowground litter pools per ground area: 
                                                                  !! @tex $(gC m^{-2})$ @endtex !!ORCHIDEE grid
	REAL(r_std), DIMENSION(nbpt,nvm,nslmd,ndoc,npool) ::doc_old  !! DOC pools per ground area:
                                                                  !! @tex $(gC m^{-2})$ @endtex !!ORCHIDEE grid
	REAL(r_std), DIMENSION(nbpt,nvm)  ::erorate         		  !! Daily average net soil loss rate (g m^{-2} dt_slow^{-1} PFT-1)
	REAL(r_std), DIMENSION(nbpt)  ::sed_deposition_tot            !! Daily total sediment deposition rate (g m^{-2} day-1)
	REAL(r_std), DIMENSION(nbpt,ncarb,nvm)  ::soc_deposition      !! Daily POC deposition rate (g C m^{-2} day-1 PFT-1)
	REAL(r_std), DIMENSION(nbpt)      :: bio_frac                 !! total wet deposition of DOC onto the flooded
	REAL(r_std), DIMENSION(nslm)      :: dly                      !! Depth of each soil layer (m)
	REAL(r_std), DIMENSION(nbpt,nlitt,nvm)           :: erolitbe  !! Eroded belowground litter (gC/m^2/day)
	REAL(r_std), DIMENSION(nbpt,nvm,ndoc,npool)      :: erodoc    !! Eroded doc (gC/m^2/day)
	REAL(r_std), DIMENSION(nbpt,nctext) 			 :: sed_text  !! Texture of sediment exported to the river (fast resvoir)
!_ ================================================================	
	
    !
	laydep(nslm) = maxsoildep
	DO ig = 1,nslm-1
		laydep(ig) = maxsoildep/(2**(nslm-1)-1)*((2**(ig-1) -1)+(2**(ig)-1))/2.0
	ENDDO
	
	dly(1)=laydep(1)
	DO ig = 2,nslm
	   dly(ig)=laydep(ig)-laydep(ig-1)
	ENDDO
	!		WRITE(numout,*) 'Depth_Soillayer', laydep(:)
	
!! 1.1 Aggregate runoff values per day. Extract the daily total runoff amount and peak runoff rate
!! peak runoff rate = maximum of the 48 half-hour runoff rates winthin a day	
	DO ig=1,nbpt
		! Only runoff considered
		meanrunoff(ig) = meanrunoff(ig) + runoff(ig)
		IF (runoff(ig) .GT. peakrunoff(ig)) THEN
			peakrunoff(ig) = runoff(ig)
		ENDIF !(runoff(ig) .GT. peakrunoff(ig))
		
		IF (runoff(ig) .GT. min_sechiba) THEN
			no_events(ig) = no_events(ig) + 1
		ENDIF !(runoff(ig) .GT. min_sechiba)
		!
		! Both runoff and drainage considered
		meanrundra(ig) = meanrundra(ig) + runoff(ig)+drainage(ig)
		IF (runoff(ig)+drainage(ig) .GT. peakrundra(ig)) THEN
			peakrundra(ig) = runoff(ig)+drainage(ig)
		ENDIF !(runoff(ig) .GT. peakrunoff(ig))
	ENDDO !ig=1,nbpt
	!
    time_counter_ero = time_counter_ero + dt_sechiba 
!	WRITE(numout,*) 'time_counter_ero',time_counter_ero,'dt_sechiba',dt_sechiba
!	WRITE(numout,*) 'runoff',runoff(1),'meanrunoff',meanrunoff(1),'peakrunoff',peakrunoff(1)
    !
	seddep_rate = zero             !! Sediment deposition rate (m day-1) for each PFT (m/day)
	totrunoff_d = zero             !! daily total runoff (mm)
	peakrunoff_d = zero            !! daily peak runoff rate.(mm/s = kg/m-2/s)
	totrundra_d  = zero            !! daily total runoff+drainage (mm)
	peakrundra_d = zero            !! daily peak (runoff+drainage) rate.(mm/s = kg/m-2/s)
	erocap = zero                  !! Erosion capacity from subgrid.(ton)
	erodep= zero                   !! Erosion depth of sub_grid (m) under different PFTs (m)
	eroc = zero                    !! Carbon export (gC/m^2/day)
	tot_erocap = zero              !! Erosion capacity from subgrid.(ton)
	ave_erodep = zero              !! Erosion depth of sub_grid (m)
	tot_eroc = zero                !! Carbon export (gC/m^2/day)
	cf_veget = zero                !! Daily vegetation cover factor
	kf_litter = zero               !! Daily aboveground litter cover factor
	kf_root = zero                 !! Daily belowground litter cover factor
	POC_EXP_agg = zero		       !! POC exports along with surface runoff, in ex $(gC m^{-2} dt_slow^{-1})$ @endtex 
	SED_EXP_agg = zero 		       !! sediment exports along with surface runoff, in @tex $(g m^{-2} dt_slow^{-1})$ @endtex
	DOC_ERO_agg = zero 		       !! DOC exports along with surface runoff, in @tex $(g m^{-2} dt_slow^{-1})$ @endtex
!! 2 If the simulation for the whole 48 half-hour intervals of a day have been finished,
!!   we calculate the erosion.
   !! 2.1 output daily mean runoff and the peak runoff rate
    !
    IF ( NINT(time_counter_ero) .GE. NINT(dt_erosion) ) THEN 
	
	    bio_frac(:)=SUM(veget_max(:,2:13),DIM=2)
		
		peakrunoff(:) = peakrunoff(:) / dt_sechiba
		peakrundra(:) = peakrundra(:) / dt_sechiba
	!!	peakrunoff(:) = peakrunoff(:)
		WHERE (peakrunoff(:) .LT. min_sechiba)
		peakrunoff(:) = zero
		ENDWHERE
		WHERE (peakrundra(:) .LT. min_sechiba)
		peakrundra(:) = zero
		ENDWHERE
	!	
		WHERE (meanrunoff(:) .LT. min_sechiba)
		meanrunoff(:) = zero
		ENDWHERE
		WHERE (meanrundra(:) .LT. min_sechiba)
		meanrundra(:) = zero
		ENDWHERE
		totrunoff_d(:)=meanrunoff(:)
		peakrunoff_d(:)=peakrunoff(:)
		totrundra_d(:)=meanrundra(:)
		peakrundra_d(:)=peakrundra(:)
	!	
	!	WRITE(numout,*) 'no of events: ', no_events(:)			
	!	WRITE(numout,*) 'peak runoff was: ', peakrunoff(:)	
	!	WRITE(numout,*) 'mean runoff was: ', meanrunoff(:)
	!	WRITE(numout,*) 'peak runoff was: ', peakrundra(:)	
	!	WRITE(numout,*) 'mean runoff was: ', meanrundra(:)
	!	WRITE(numout,*) 'Erosion_bulkdens: ', bulkdens(:)
	!   WRITE(numout,*) 'Erosion_ZHC1'
	!   WRITE(numout,*) 'litt_ab: ', MAXVAL(litter_above),MINVAL(litter_above)
	!   WRITE(numout,*) 'litt_be: ', MAXVAL(litter_below),MINVAL(litter_below)
	!   WRITE(numout,*) 'ligninbe',MAXVAL(lignin_struc_below),MINVAL(lignin_struc_below)
	!   WRITE(numout,*) 'DOC',MAXVAL(DOC),MINVAL(DOC)
	!   WRITE(numout,*) 'SOC',MAXVAL(carbon),MINVAL(carbon)
	!	WRITE(numout,*) 'layer_depth1: ', laydep(7)
	!
    ! 2.2 Calculate vegetation, litter, root factor
	! 2.2.1 Calculate vegetation and management factor
		cf_veget(:,:)=0.0
		cveg_f(:,:)=0.0
		cf_veget(:,1)=zero
		cveg_f(:,1)=zero
		
		DO jg=1,nbpt
			cveg_f(jg,1)=0.0
			DO lg=2,nvm
			  IF (veget_max(jg,lg).GT.zero) THEN
				vegf=100.0*veget(jg,lg)/veget_max(jg,lg)
				IF (vegf.LT. 0.11) THEN
					cveg_f(jg,lg) = 1.0
				ELSEIF (vegf .GT. 95.) THEN
					cveg_f(jg,lg) = min_cvegf
				ELSE
					cveg_f(jg,lg) = cvegf_coef1-cvegf_coef2*log10(vegf)
				ENDIF !(veget(jg,2:nvm) .LT. 0.11)
			  ENDIF
			ENDDO !DO lg=2,nvm
		ENDDO !jg=1,nbpt
		cf_veget(:,:)=cveg_f(:,:)
	! 2.2.2 Calculate factors of aboveground litter and belowground litter
		DO jg=1,nbpt
		    kflitter(jg,1)=0.0
		    kfroot(jg,1)=0.0
		    DO lg=2,nvm
				IF (lg.LT.12) THEN
					littotal=0.001*(sum(litter_above(jg,:,lg,icarbon))+sum(litter_below(jg,:,lg,1:7,icarbon)))
				ELSE
					littotal=0.001*sum(litter_below(jg,:,lg,1:7,icarbon))
				ENDIF
				!
				IF (veget_max(jg,lg).GT.zero) THEN
					kflitter(jg,lg)=EXP(-littf_exponent*littotal)
				ELSE
					kflitter(jg,lg)=0.0
				ENDIF
				!
				belowbiom=0.001*(rootmass(jg,lg,isapbelow,icarbon)+ &
					&	rootmass(jg,lg,iheartbelow,icarbon)+rootmass(jg,lg,iroot,icarbon))
		        IF (veget_max(jg,lg).GT.zero) THEN
					kfroot(jg,lg)=EXP(-rootf_exponent*belowbiom)
				ELSE
					kfroot(jg,lg)=0.0
				ENDIF
				!WRITE(numout,*) 'ZHC2', jg,lg,littotal,kflitter(jg,lg)
		    ENDDO !lg=2,nvm
		ENDDO !jg=1,nbpt
		kf_litter(:,:)=kflitter(:,:)
		kf_root(:,:)=kfroot(:,:)
    !
    !! 2.3 Calculate erosion rate based on MUSLE equation
	!    2.3.1  Calculate the sediment amount(ton) for each finer(sub-) grid (standard erosion grid)
		erocap(:,:)=zero
		tot_erocap(:)=zero
		erodep(:,:)=zero
		ave_erodep(:)=zero
		erorate(:,:)=zero
		
		DO ig=1,nbpt
			meanrun_up=totrunoff_d(ig)!
			peakrun_up=peakrunoff_d(ig)!
			IF (temp_sol(ig).LT.(273.15-5.0)) THEN ! H. Zhang: When soil temperature is lower than -5-degree, we assumed no erosion occure 
				erocap(ig,:)=zero
				erodep(ig,:)=zero
				tot_erocap(ig)=zero
				ave_erodep(ig)=zero
			ELSE
				DO lg=2,nvm
					IF (veget_max(ig,lg).GT.zero) THEN
						! Only surface runoff considered
						cveg_up=cf_veget(ig,lg)
						kflitter_up=kf_litter(ig,lg)
						kfroot_up=kf_root(ig,lg)
						erocap(ig,lg)=(meanrun_up*peakrun_up/(std_runoff*std_qp)/1000.)**runf_power &
						&        *(cveg_up/std_cf)*std_totsed(ig)*kflitter_up*kfroot_up*veget_max(ig,lg)    !*kfroot_up   % unit ton/day (Mg/day)
						! surface runoff+drainage
						!	erocap(ig)=(meanrundra_up*peakrundra_up/(std_runoff*std_qp)/1000)**0.56 &
						!     &        *(cveg_up/std_cf)*std_totsed(ig)*kflitter_up    !*kfroot_up
						IF (erocap(ig,lg).LT.zero) THEN
							erocap(ig,lg)=zero
						ENDIF
						! depth of the sediment if all input sediments from upstream are deposited at one grid-cell
						! Although soil erosion only occurs in the effective part. the erosion rate/depth is averaged to the whole grid cell
						IF (gridarea(ig).GT.zero.AND.bulkdens(ig).GT.zero) THEN
							erorate(ig,lg)=1.0E+6*erocap(ig,lg)/(gridarea(ig)*veget_max(ig,lg)) ! g soil /m^2 /day ##*effgrid_perc(ig)
							erodep(ig,lg)=1000.*erocap(ig,lg)/(gridarea(ig)*veget_max(ig,lg))/bulkdens(ig) !m/day  ## *effgrid_perc(ig)
							!WRITE(numout,*) 'ZHC3_ero', ig,lg,effgrid_perc(ig),bulkdens(ig),erodep(ig,lg),erorate(ig,lg),std_totsed(ig)
						ENDIF
					!
					ENDIF !(veget_max(ig,lg).GT.zero) THEN
				ENDDO !DO lg=1,nvm
				! Average erosion rate for each grid-cell
				tot_erocap(ig)=SUM(erocap(ig,:)) ! ton/day
				IF (gridarea(ig).GT.zero.AND.bulkdens(ig).GT.zero) THEN
				    ave_erodep(ig)=1000.*tot_erocap(ig)/(bio_frac(ig)*gridarea(ig))/bulkdens(ig) ! m/day ## *effgrid_perc(ig)
				ENDIF
			ENDIF !IF (temp_sol(ig).LT.(-2.0)) THEN
		ENDDO !DO lg=1,nbpt
		!WRITE(numout,*) 'erosion depth', SUM(ave_erodep(:))
		!
		!2.4 Calculate the processes of erosion rate of SOC
		!      Calculate from the upstream to outlets of the watershed based on flow accumulation
		!WRITE(numout,*) 'erosion_carbon', carbon(:,1,:,1)
		tot_eroc(:,:)=zero
		!
		carbon_old(:,:,:,:)=carbon(:,:,:,:)
		litbe_old(:,:,:,:)=litter_below(:,:,:,:,icarbon)
		litbe_tot(:,:,:)=SUM(litbe_old(:,:,:,:),DIM=2)
		doc_old(:,:,:,:,:)=doc(:,:,:,:,:,icarbon)
		!
		eroc(:,:,:)=zero
		erolitbe(:,:,:)=zero
		erodoc(:,:,:,:)=zero
		!
		tot_7c(:,:,:)=zero
		tot_7litbelow(:,:,:)=zero
		tot_7doc(:,:,:,:)=zero
		
		!WRITE(numout,*) 'SUMero1_litter',SUM(litter_above),SUM(litter_below)
	    !WRITE(numout,*) 'SUMero1_SOC',SUM(carbon)
	    !WRITE(numout,*) 'SUMero1_DOC',SUM(DOC)
		!WRITE(numout,*) 'ligninbe',SUM(lignin_struc_below)
		!WRITE(numout,*) 'laydep',laydep(:)
		!WRITE(numout,*) 'dly',dly(:)
		
	    DO ig=1,nbpt
		    !WRITE(numout,*) 'SUMero1',ig,SUM(litter_below(ig,:,1,:,icarbon)),SUM(carbon(ig,:,1,:)), &
			!                 lignin_struc_below(ig,1,:),SUM(DOC(ig,1,:,:,:,icarbon))
			DO lg=2,nvm
			    !WRITE(numout,*) 'SUMero2',ig,lg,SUM(litter_below(ig,:,lg,:,icarbon)),SUM(DOC(ig,1,:,:,:,icarbon))
				
				erodep(ig,lg)=MIN(erodep(ig,lg),laydep(7)) ! H. Zhang: depth of daily eroded soil will not exceed laydep(7) (20 cm)
				
				IF (ISNAN(SUM(carbon(ig,3,lg,:)))) THEN
					carbon(ig,:,lg,:)=zero
					carbon_old(ig,:,lg,:)=zero
				ENDIF !
				!
				IF ((veget_max(ig,lg) .GT. 0).AND.(erodep(ig,lg).GT.0).AND.(MinVal(SUM(carbon(ig,:,lg,1:7),DIM=2)).GT.0)) THEN  !!! 1<=fldir<=128
				    DO jg=1,7
					   tot_7c(ig,:,lg)=tot_7c(ig,:,lg)+carbon(ig,:,lg,jg)
					   tot_7litbelow(ig,:,lg)=tot_7litbelow(ig,:,lg)+litter_below(ig,:,lg,jg,icarbon)
					!   tot_7doc(ig,lg,:,:)=tot_7doc(ig,lg,:,:)+DOC(ig,lg,jg,:,:,icarbon)
					   lignin_ave= lignin_ave+lignin_struc_below(ig,lg,jg)*SUM(litter_below(ig,:,lg,jg,icarbon))
					ENDDO !DO jg=,7
					eroc(ig,:,lg) = erodep(ig,lg)/laydep(7)*tot_7c(ig,:,lg)
					erolitbe(ig,:,lg) = erodep(ig,lg)/laydep(7)*tot_7litbelow(ig,:,lg)
					!erodoc(ig,lg,:,:) = erodep(ig,lg)/laydep(7)*tot_7doc(ig,lg,:,:)
					
					tot_eroc(ig,:)=tot_eroc(ig,:)+ eroc(ig,:,lg)*veget_max(ig,lg)
					
					!! H. Zhang. Here we assumed that the soil erosion only takes sediment, POC and DOC into rivers. 
					!! Belowground litter in the eroded soil layer will enter into the surface litter pools
					IF (SUM(tot_7litbelow(ig,:,lg)).GT.0) THEN
					    lignin_ave=lignin_ave/SUM(tot_7litbelow(ig,:,lg))
				!	    lignin_struc_above(ig,lg)= (lignin_struc_above(ig,lg)*SUM(litter_above(ig,:,lg,icarbon))+ &
				!	        lignin_ave*SUM(erolitbe(ig,:,lg)))/(SUM(litter_above(ig,:,lg,icarbon))+SUM(erolitbe(ig,:,lg)))
					    litter_above(ig,:,lg,icarbon)=litter_above(ig,:,lg,icarbon) + erolitbe(ig,:,lg)
					ENDIF
					
					DO jg=1,7 ! top 7 soil layers
					  WHERE (tot_7c(ig,:,lg).GT.zero)
						carbon(ig,:,lg,jg)=carbon_old(ig,:,lg,jg)-eroc(ig,:,lg)*carbon_old(ig,:,lg,jg)/tot_7c(ig,:,lg)+ &
						&		(erodep(ig,lg)/dly(8)*carbon_old(ig,:,lg,8))*carbon_old(ig,:,lg,jg)/tot_7c(ig,:,lg)
					  ENDWHERE
					  
					  IF (SUM(tot_7litbelow(ig,:,lg)).GT.zero) THEN
					    WHERE (tot_7litbelow(ig,:,lg).GT.zero)
						  litter_below(ig,:,lg,jg,icarbon)=litbe_old(ig,:,lg,jg)-erolitbe(ig,:,lg)*litbe_old(ig,:,lg,jg)/tot_7litbelow(ig,:,lg)+ &
						       &		(erodep(ig,lg)/dly(8)*litbe_old(ig,:,lg,8))*litbe_old(ig,:,lg,jg)/tot_7litbelow(ig,:,lg)
						ENDWHERE
				!		lignin_struc_below(ig,lg,jg)= (lignin_struc_below(ig,lg,jg)*(litbe_tot(ig,lg,jg)- &
				!		&    SUM(erolitbe(ig,:,lg))*litbe_tot(ig,lg,jg)/SUM(tot_7litbelow(ig,:,lg)))+ &
				!		&    lignin_struc_below(ig,lg,8)*(erodep(ig,lg)/dly(8)*litbe_tot(ig,lg,8))*litbe_tot(ig,lg,jg)/SUM(tot_7litbelow(ig,:,lg))) / &
				!		&    SUM(litter_below(ig,:,lg,jg,icarbon))
					  ENDIF
					  
					!  WHERE(tot_7doc(ig,lg,:,:).GT.zero)
					!    DOC(ig,lg,jg,:,:,icarbon)=doc_old(ig,lg,jg,:,:)-erodoc(ig,lg,:,:)*doc_old(ig,lg,jg,:,:)/tot_7doc(ig,lg,:,:) + &
					!	&      (erodep(ig,lg)/dly(8)* doc_old(ig,lg,8,:,:))*doc_old(ig,lg,jg,:,:)/tot_7doc(ig,lg,:,:)
					!  ENDWHERE
					  
					!  WRITE(numout,*) 'SUMero3',ig,lg,SUM(litter_below(ig,:,lg,:,icarbon)),SUM(carbon(ig,:,lg,jg)),  &
					!                  lignin_struc_below(ig,lg,jg),SUM(DOC(ig,lg,jg,:,:,icarbon))
					ENDDO ! top 7 soil layers
					DO jg=nslm-1,8,-1 ! top 8-10 soil layers
						carbon(ig,:,lg,jg)= carbon_old(ig,:,lg,jg)-erodep(ig,lg)/dly(jg)*carbon_old(ig,:,lg,jg)+ &
						&			erodep(ig,lg)/dly(jg+1)*carbon_old(ig,:,lg,jg+1)
					    litter_below(ig,:,lg,jg,icarbon)=litbe_old(ig,:,lg,jg)-erodep(ig,lg)/dly(jg)*litbe_old(ig,:,lg,jg) + &
						&           erodep(ig,lg)/dly(jg+1)*litbe_old(ig,:,lg,jg+1)
				!		IF (SUM(litter_below(ig,:,lg,jg,icarbon)).GT.zero) THEN
				!		   lignin_struc_below(ig,lg,jg)= (lignin_struc_below(ig,lg,jg)*(litbe_tot(ig,lg,jg)-erodep(ig,lg)/dly(jg)*litbe_tot(ig,lg,jg)) + &
				!		   &         lignin_struc_below(ig,lg,jg+1)*erodep(ig,lg)/dly(jg+1)*litbe_tot(ig,lg,jg+1)) / SUM(litter_below(ig,:,lg,jg,icarbon))
				!		ENDIF
					!	DOC(ig,lg,jg,:,:,icarbon)= doc_old(ig,lg,jg,:,:)-erodep(ig,lg)/dly(jg)*doc_old(ig,lg,jg,:,:) + &
					!	&           erodep(ig,lg)/dly(jg+1)*doc_old(ig,lg,jg+1,:,:)
						
					!	WRITE(numout,*) 'SUMero4',ig,lg,SUM(litter_below(ig,:,lg,:,icarbon)),SUM(carbon(ig,:,lg,jg)),  &
					!                  lignin_struc_below(ig,lg,jg),SUM(DOC(ig,lg,jg,:,:,icarbon))
					ENDDO
					!
					IF (depth_deepsoil(ig,lg).GT.min_sechiba .AND.depth_deepsoil(ig,lg).GT.erodep(ig,lg)) THEN
						carbon(ig,:,lg,nslm)=carbon_old(ig,:,lg,nslm)-erodep(ig,lg)/dly(nslm)*carbon_old(ig,:,lg,nslm) + &
						&                   erodep(ig,lg)/depth_deepsoil(ig,lg)*carbon_old(ig,:,lg,nslmd)
						carbon(ig,:,lg,nslmd)=(1.0-erodep(ig,lg)/depth_deepsoil(ig,lg))*carbon_old(ig,:,lg,nslmd)
						
						litter_below(ig,:,lg,nslm,icarbon)=litbe_old(ig,:,lg,nslm)-erodep(ig,lg)/dly(nslm)*litbe_old(ig,:,lg,nslm) +  &
						&    erodep(ig,lg)/depth_deepsoil(ig,lg)*litbe_old(ig,:,lg,nslmd)
						litter_below(ig,:,lg,nslmd,icarbon)=(1.0-erodep(ig,lg)/depth_deepsoil(ig,lg))*litbe_old(ig,:,lg,nslmd)
						IF (SUM(litter_below(ig,:,lg,nslm,icarbon)).GT.zero) THEN
				!		lignin_struc_below(ig,lg,nslm)=(lignin_struc_below(ig,lg,nslm)*(litbe_tot(ig,lg,nslm)-erodep(ig,lg)/dly(nslm)* &
				!		&    litbe_tot(ig,lg,nslm))+erodep(ig,lg)/depth_deepsoil(ig,lg)*litbe_tot(ig,lg,nslmd)* &
				!		&    lignin_struc_below(ig,lg,nslmd))/SUM(litter_below(ig,:,lg,nslm,icarbon))
						ENDIF
					!	DOC(ig,lg,nslm,:,:,icarbon)=doc_old(ig,lg,nslm,:,:)-erodep(ig,lg)/dly(nslm)*doc_old(ig,lg,nslm,:,:) + &
					!	&           erodep(ig,lg)/depth_deepsoil(ig,lg)*doc_old(ig,lg,nslmd,:,:)
					!	DOC(ig,lg,nslmd,:,:,icarbon)=(1.0-erodep(ig,lg)/depth_deepsoil(ig,lg))*doc_old(ig,lg,nslmd,:,:)
						
					!	WRITE(numout,*) 'SUMero5',ig,lg,SUM(litter_below(ig,:,lg,nslm:nslmd,icarbon)),SUM(carbon(ig,:,lg,nslm:nslmd)),  &
					!                  lignin_struc_below(ig,lg,nslm),lignin_struc_below(ig,lg,nslmd),SUM(DOC(ig,lg,nslm:nslmd,:,:,icarbon))
						! kjpindex,nvm,nslmd,ndoc,npool,nelements
					ELSEIF (depth_deepsoil(ig,lg).LE.erodep(ig,lg)) THEN
						carbon(ig,:,lg,nslm)=carbon_old(ig,:,lg,nslm)-erodep(ig,lg)/dly(nslm)*carbon_old(ig,:,lg,nslm) + &
						&                   carbon_old(ig,:,lg,nslmd)
						carbon(ig,:,lg,nslmd)=zero

						litter_below(ig,:,lg,nslm,icarbon)=litbe_old(ig,:,lg,nslm)-erodep(ig,lg)/dly(nslm)*litbe_old(ig,:,lg,nslm) + &
						&    litbe_old(ig,:,lg,nslmd)
						litter_below(ig,:,lg,nslmd,icarbon)= zero
				!		IF (SUM(litter_below(ig,:,lg,jg,icarbon)).GT.zero) THEN
				!		  lignin_struc_below(ig,lg,nslm)=(lignin_struc_below(ig,lg,nslm)*(litbe_tot(ig,lg,nslm)-erodep(ig,lg)/dly(nslm)* &
				!		    &    litbe_tot(ig,lg,nslm))+litbe_tot(ig,lg,nslmd)*lignin_struc_below(ig,lg,nslmd))/ &
				!		    &    SUM(litter_below(ig,:,lg,nslm,icarbon))
				!		ENDIF
						lignin_struc_below(ig,lg,nslmd)=zero
						
					!	DOC(ig,lg,nslm,:,:,icarbon)=doc_old(ig,lg,nslm,:,:)-erodep(ig,lg)/dly(nslm)*doc_old(ig,lg,nslm,:,:) + &
					!	&           doc_old(ig,lg,nslmd,:,:)
					!	DOC(ig,lg,nslmd,:,:,icarbon)=zero
						
					!	WRITE(numout,*) 'SUMero6',ig,lg,SUM(litter_below(ig,:,lg,nslm:nslmd,icarbon)),SUM(carbon(ig,:,lg,nslm:nslmd)),  &
					!                  lignin_struc_below(ig,lg,nslm),lignin_struc_below(ig,lg,nslmd),SUM(DOC(ig,lg,nslm:nslmd,:,:,icarbon))
					ELSE
					    carbon(ig,:,lg,nslm)=carbon_old(ig,:,lg,nslm)-erodep(ig,lg)/dly(nslm)*carbon_old(ig,:,lg,nslm)
						carbon(ig,:,lg,nslmd)=zero

						litter_below(ig,:,lg,nslm,icarbon)=litbe_old(ig,:,lg,nslm)-erodep(ig,lg)/dly(nslm)*litbe_old(ig,:,lg,nslm)
						litter_below(ig,:,lg,nslmd,icarbon)= zero
						lignin_struc_below(ig,lg,nslmd)=zero
						
					!	DOC(ig,lg,nslm,:,:,icarbon)=doc_old(ig,lg,nslm,:,:)-erodep(ig,lg)/dly(nslm)*doc_old(ig,lg,nslm,:,:)
					!	DOC(ig,lg,nslmd,:,:,icarbon)=zero
						
					!	WRITE(numout,*) 'SUMero7',ig,lg,SUM(litter_below(ig,:,lg,nslm:nslmd,icarbon)),SUM(carbon(ig,:,lg,nslm:nslmd)),  &
					!                  lignin_struc_below(ig,lg,nslm),lignin_struc_below(ig,lg,nslmd),SUM(DOC(ig,lg,nslm:nslmd,:,:,icarbon))
					ENDIF
					depth_deepsoil(ig,lg)=MAX(zero,depth_deepsoil(ig,lg)-erodep(ig,lg))
					
				!	WRITE(numout,*) 'soilcarbon_passive', carbon(ig,3,lg,1:7)
				!	WRITE(numout,*) 'soilcarbon_active', carbon(ig,1,lg,1:7)	
				ENDIF
				!WRITE(numout,*) 'ero_totc', ig,lg,SUM(litter_below(ig,:,lg,:,icarbon)),SUM(litter_below(ig,:,1,:,icarbon)), &
				!& SUM(lignin_struc_below(ig,lg,:))
                        !        WRITE(numout,*) 'ero_totc', veget_max(ig,lg),MinVal(carbon(ig,:,lg,1:7)),SUM(tot_eroc(ig,:))
			ENDDO !DO lg=1,nvm
		!	WRITE(numout,*) 'erosion carbon', SUM(tot_eroc(ig,:))
	    ENDDO !DO ig=1,nbpt
		eroc_active(:,:)=eroc(:,iactive,:)
		eroc_slow(:,:)=eroc(:,islow,:)
		eroc_passive(:,:)=eroc(:,ipassive,:)
		
		!WRITE(numout,*) 'SUMero2_litter',MAXVAL(litter_above),MAXVAL(litter_below)
	    !WRITE(numout,*) 'SUMero2_SOC',MAXVAL(carbon)
	    !WRITE(numout,*) 'SUMero2_DOC',MAXVAL(DOC)
		
    ! 2.5 Average export rate of sediment and POC from land to river network
	    SED_EXP_agg(:,:)=zero
		POC_EXP_agg(:,:)=zero
		DOC_ERO_agg(:,:)=zero
		sed_text(:,:)=zero
		
	! *** Assumption-1: the enrichment ratio of clay fraction is enrich_clay=3
		! In case the sum of fractions of clay, silt, sand are less than un
		sed_text(:,ic_clay)=MAX(zero,un-textfrac(:,ic_silt)-textfrac(:,ic_sand))
		
		sed_text(:,ic_clay)=MAX(sed_text(:,ic_clay),MIN(un,sed_text(:,ic_clay)*enrich_clay))
		sed_text(:,ic_silt)=MIN(textfrac(:,ic_silt)*enrich_silt,un-sed_text(:,ic_clay))
		sed_text(:,ic_sand)=MIN(textfrac(:,ic_sand),un-sed_text(:,ic_clay)-sed_text(:,ic_silt))
	
	! *** Assumption-2: All sediment transported into river channel is the clay fraction; Or we just simulate the 
	!     total sediment transport, without distinguish the clay, silt and sand fraction
	!    sed_text(:,ic_clay)= un
	!	sed_text(:,ic_silt)= zero
	!	sed_text(:,ic_sand)= zero
	
		DO lg=1,nvm
			!WHERE(erorate(:,lg).GT.zero)
			SED_EXP_agg(:,ic_clay)=SED_EXP_agg(:,ic_clay)+ &
				erorate(:,lg)*sed_text(:,ic_clay)*veget_max(:,lg)
					
			SED_EXP_agg(:,ic_silt)=SED_EXP_agg(:,ic_silt)+ &
				erorate(:,lg)*sed_text(:,ic_silt)*veget_max(:,lg)
					
			SED_EXP_agg(:,ic_sand)=SED_EXP_agg(:,ic_sand)+ &
				erorate(:,lg)*sed_text(:,ic_sand)*veget_max(:,lg)
			!ENDWHERE
			!
			DO icarb=1,ncarb
			!	WHERE(eroc(:,icarb,lg).GT.zero)
				POC_EXP_agg(:,icarb)=POC_EXP_agg(:,icarb)+eroc(:,icarb,lg)*veget_max(:,lg)
			!	ENDWHERE
			ENDDO
			!
			DO ii = 1,ndoc
				DOC_ERO_agg(:,iactive) = DOC_ERO_agg(:,iactive)+ &
				    (erodoc(:,lg,ii,imetabo)+erodoc(:,lg,ii,imetbel)+erodoc(:,lg,ii,iact))*veget_max(:,lg)
				DOC_ERO_agg(:,islow) = DOC_ERO_agg(:,islow)+ &
				    (erodoc(:,lg,ii,istrabo)+erodoc(:,lg,ii,istrbel)+erodoc(:,lg,ii,islo))*veget_max(:,lg)
				DOC_ERO_agg(:,ipassive) = DOC_ERO_agg(:,ipassive)+ &
					erodoc(:,lg,ii,ipas)*veget_max(:,lg)
			ENDDO
			!
		ENDDO !DO lg=1,nvm
		!SED_EXP_agg(:,:)=SED_EXP_agg(:,:)
		!POC_EXP_agg(:,:)=POC_EXP_agg(:,:)
		!DOC_ERO_agg(:,:)=DOC_ERO_agg(:,:)
		
		!WRITE(numout,*) 'ZHCero_Cexp0'
		!DO ig=1,nbpt
		!	WRITE(numout,*) 'Runoff', ig,totrunoff_d(ig)
		!	WRITE(numout,*) 'effgrid_perc',effgrid_perc(ig)
		!	WRITE(numout,*) 'std_totsed',std_totsed(ig)
		!	WRITE(numout,*) 'erocap_pft',ig,SUM(erocap(ig,:))
		!	WRITE(numout,*) 'effgrid_perc',effgrid_perc(ig)
		!	WRITE(numout,*) 'Erorate', erorate(ig,:)
		!	WRITE(numout,*) 'textfrac',textfrac(ig,:),sed_text(ig,:)
		!	WRITE(numout,*) 'Sediment', SED_EXP_agg(ig,:)
		!	WRITE(numout,*) 'POC', POC_EXP_agg(ig,:)*one_day/dt_sechiba
		!	WRITE(numout,*) 'Concent_g/m^3', SED_EXP_agg(ig,:)*one_day/dt_sechiba/totrunoff_d(ig)*1000.0
		!ENDDO
    !! 2.6 Update SOC, litter_below and DOC pools caused by sediment deposition
	seddep_rate(:,:)=zero
	soc_deposition(:,:,:)=zero
	
	sed_deposition_tot(:) = (sed_deposition_d(:,ic_clay)+sed_deposition_d(:,ic_silt)+ &
							& sed_deposition_d(:,ic_sand))*dt_erosion/one_day
	poc_deposition_d(:,:) = poc_deposition_d(:,:)*dt_erosion/one_day
	
	max_depth=min(laydep(7),dly(8))
	DO lg=2,nvm
	   WHERE (bio_frac(:).GT.zero.AND.bulkdens(:).GT.zero)
		   seddep_rate(:,lg) = sed_deposition_tot(:)/bio_frac(:)/bulkdens(:)
	   ENDWHERE
	   ! H.Zhang: We assumed that 1) the deposited sediemnt and POC will evenly distributed in 
	   ! each grid-cell, 2) the sediment and POC are fully mixed; 3) the depth of daily sediment 
	   ! deopisiton does not exceed laydep(4) (~5.0 cm)
	   ! This assumption did affect the C balance
	   WHERE (seddep_rate(:,lg).GT.max_depth)
		   seddep_rate(:,lg) = max_depth-min_sechiba
	   ENDWHERE
	   depth_deepsoil(:,lg)=depth_deepsoil(:,lg)+seddep_rate(:,lg) ! unit: m
	   !
	   DO icarb=1,ncarb
		   WHERE (bio_frac(:).GT.zero.AND.veget_max(:,lg).GT.zero)
               soc_deposition(:,icarb,lg)=1000.*poc_deposition_d(:,icarb)/bio_frac(:) ! Unit: g C m-2 day-1
		   ENDWHERE   
	   ENDDO !icarb=1,ncarb
	ENDDO !! DO lg=2,nvm
	
	carbon_old(:,:,:,:)=carbon(:,:,:,:)
	litbe_old(:,:,:,:)=litter_below(:,:,:,:,icarbon)
	litbe_tot(:,:,:)=SUM(litbe_old(:,:,:,:),DIM=2)
	doc_old(:,:,:,:,:)=doc(:,:,:,:,:,icarbon)
	tot_7c(:,:,:)=zero
	tot_7litbelow(:,:,:)=zero
	tot_7doc(:,:,:,:)=zero
	
	DO ig=1,nbpt
	  DO lg=2,nvm
	    !
	    !WRITE(numout,*) 'ZHC_below0', ig,lg,seddep_rate(ig,lg)
		IF ((veget_max(ig,lg) .GT. 0).AND.(seddep_rate(ig,lg).GT.zero).AND.(MinVal(SUM(carbon(ig,:,lg,1:7),DIM=2)).GT.0)) THEN  !!! 1<=fldir<=128
			DO jg=1,7
				tot_7c(ig,:,lg)=tot_7c(ig,:,lg)+carbon(ig,:,lg,jg)
				tot_7litbelow(ig,:,lg)=tot_7litbelow(ig,:,lg)+litter_below(ig,:,lg,jg,icarbon)
				tot_7doc(ig,lg,:,:)=tot_7doc(ig,lg,:,:)+DOC(ig,lg,jg,:,:,icarbon)
			ENDDO !DO jg=,7
			! Situation-1: Part of original layer-1 enter layer-2; newly deposited soil does not reach layer-2
		   carbon(ig,:,lg,nslmd)=carbon_old(ig,:,lg,nslmd)+seddep_rate(ig,lg)/dly(nslm)*carbon_old(ig,:,lg,nslm)
		   litter_below(ig,:,lg,nslmd,icarbon)=litbe_old(ig,:,lg,nslmd)+seddep_rate(ig,lg)/dly(nslm)*litbe_old(ig,:,lg,nslm)
		   IF (SUM(litter_below(ig,:,lg,nslmd,icarbon)).GT.zero) THEN
		       lignin_struc_below(ig,lg,nslmd)=(litbe_tot(ig,lg,nslmd)*lignin_struc_below(ig,lg,nslmd)+ &
		       &     seddep_rate(ig,lg)/dly(nslm)*litbe_tot(ig,lg,nslm)*lignin_struc_below(ig,lg,nslm))/  &
		       &	SUM(litter_below(ig,:,lg,nslmd,icarbon))
		   ENDIF
		   DOC(ig,lg,nslmd,:,:,icarbon)=doc_old(ig,lg,nslmd,:,:)+seddep_rate(ig,lg)/dly(nslm)*doc_old(ig,lg,nslm,:,:)
		   !
		   DO jg=nslm,9,-1
			  carbon(ig,:,lg,jg)=(dly(jg)-seddep_rate(ig,lg))/dly(jg)*carbon_old(ig,:,lg,jg)+seddep_rate(ig,lg)/laydep(jg-1)*carbon_old(ig,:,lg,jg-1)
			  litter_below(ig,:,lg,jg,icarbon)=(dly(jg)-seddep_rate(ig,lg))/dly(jg)*litbe_old(ig,:,lg,jg)+seddep_rate(ig,lg)/laydep(jg-1)*litbe_old(ig,:,lg,jg-1)
		!	  IF (SUM(litter_below(ig,:,lg,jg,icarbon)).GT.zero) THEN
		!	    lignin_struc_below(ig,lg,jg)=(lignin_struc_below(ig,lg,jg)*(dly(jg)-seddep_rate(ig,lg))/dly(jg)*litbe_tot(ig,lg,jg)+ &
		!	      &    lignin_struc_below(ig,lg,jg-1)*seddep_rate(ig,lg)/laydep(jg-1)*litbe_tot(ig,lg,jg-1))/SUM(litter_below(ig,:,lg,jg,icarbon))
		!	  ENDIF
			  DOC(ig,lg,jg,:,:,icarbon)=(dly(jg)-seddep_rate(ig,lg))/dly(jg)*doc_old(ig,lg,jg,:,:)+seddep_rate(ig,lg)/laydep(jg-1)*doc_old(ig,lg,jg-1,:,:)
		   ENDDO !  DO jg=nslm,2,-1
		   carbon(ig,:,lg,8)=(dly(8)-seddep_rate(ig,lg))/dly(8)*carbon_old(ig,:,lg,8)+seddep_rate(ig,lg)/laydep(7)*tot_7c(ig,:,lg)
		   litter_below(ig,:,lg,8,icarbon)=(dly(8)-seddep_rate(ig,lg))/dly(8)*litbe_old(ig,:,lg,8)+seddep_rate(ig,lg)/laydep(7)*tot_7litbelow(ig,:,lg)
		   DOC(ig,lg,8,:,:,icarbon)=(dly(8)-seddep_rate(ig,lg))/dly(8)*doc_old(ig,lg,8,:,:)+seddep_rate(ig,lg)/laydep(7)*tot_7doc(ig,lg,:,:)
		   !
		   DO jg=7,1,-1
		      WHERE (tot_7c(ig,:,lg).GT.zero)
				carbon(ig,:,lg,1) = ((un-seddep_rate(ig,lg)/laydep(7))*tot_7c(ig,:,lg)+soc_deposition(ig,:,lg))*(carbon_old(ig,:,lg,jg)/tot_7c(ig,:,lg))
			  ENDWHERE
		      litter_below(ig,:,lg,jg,icarbon) = (un-seddep_rate(ig,lg)/laydep(7))*litbe_old(ig,:,lg,jg) ! No litterfall depositioon
		      DOC(ig,lg,jg,:,:,icarbon) = (un-seddep_rate(ig,lg)/laydep(7))*doc_old(ig,lg,jg,:,:)        ! DOC will enter into the nslm soil layer following water flow
		   ENDDO		   
		   !WRITE(numout,*) 'ZHC_below10', ig,lg,SUM(litter_below(ig,:,lg,nslm,icarbon)),SUM(lignin_struc_below(ig,lg,:))
		ENDIF
		
		! Make sure the lignin_struc_ is smaller than 1.0 and higher than 0.0
		lignin_struc_above(ig,lg)=MIN(MAX(lignin_struc_above(ig,lg),zero),1.0)
		DO jg=nslmd,1,-1
		   lignin_struc_below(ig,lg,jg) = MIN(MAX(lignin_struc_below(ig,lg,jg),zero),1.0)
		ENDDO
		
	  ENDDO !! DO lg=2,nvm
	ENDDO !DO ig=1,nbpt
	
	!WRITE(numout,*) 'SUMero3_litter',SUM(litter_above),SUM(litter_below),SUM(lignin_struc_below)
	!WRITE(numout,*) 'SUMero3_SOC',SUM(carbon)
	!WRITE(numout,*) 'SUMero3_DOC',SUM(DOC)
	!! 2.7 Write soil & carbon erosion rate to the history file
	
	CALL xios_orchidee_send_field("bulkdens",bulkdens*one_day/dt_erosion) ! dt_erosion=one_day
    CALL xios_orchidee_send_field("totrunoff_d",totrunoff_d*one_day/dt_erosion)
	CALL xios_orchidee_send_field("peakrunoff_d",peakrunoff_d)
	CALL xios_orchidee_send_field("totrundra_d",totrundra_d*one_day/dt_erosion)
	CALL xios_orchidee_send_field("peakrundra_d",peakrundra_d)
	CALL xios_orchidee_send_field("cf_veget",cf_veget)
	CALL xios_orchidee_send_field("kf_litter",kf_litter)
	CALL xios_orchidee_send_field("kf_root",kf_root)
	CALL xios_orchidee_send_field("erocap",erocap*one_day/dt_erosion)
	CALL xios_orchidee_send_field("erodep",erodep*one_day/dt_erosion)
	CALL xios_orchidee_send_field("eroc_active",eroc_active*one_day/dt_erosion)
	CALL xios_orchidee_send_field("eroc_slow",eroc_slow*one_day/dt_erosion)
	CALL xios_orchidee_send_field("eroc_passive",eroc_passive*one_day/dt_erosion)
	CALL xios_orchidee_send_field("tot_erocap",tot_erocap*one_day/dt_erosion)
	CALL xios_orchidee_send_field("ave_erodep",ave_erodep*one_day/dt_erosion)
	CALL xios_orchidee_send_field("tot_eroc",tot_eroc*one_day/dt_erosion)
	CALL xios_orchidee_send_field("depth_deepsoil",depth_deepsoil*one_day/dt_erosion)
	CALL xios_orchidee_send_field("seddep_rate",seddep_rate*one_day/dt_erosion)
	!	
		! Reinitialize values
		time_counter_ero = zero
		cveg_f = zero
		cf_veget=zero
		kflitter=zero
		kfroot=zero
		kf_litter = zero
		kf_root = zero
		peakrunoff = zero
		meanrunoff = zero
		peakrundra = zero
		meanrundra = zero
		erocap = zero
		erodep = zero
		eroc=zero
		tot_erocap=zero
		ave_erodep =zero
		tot_eroc =zero
		sed_deposition_d = zero
		sed_deposition_tot= zero
	    poc_deposition_d = zero
		no_events = 0
		
	!   WRITE(numout,*) 'Erosion_ZHC100'
	!   WRITE(numout,*) 'litt_ab: ', MAXVAL(litter_above),MINVAL(litter_above)
	!   WRITE(numout,*) 'litt_be: ', MAXVAL(litter_below),MINVAL(litter_below)
	!   WRITE(numout,*) 'ligninbe',MAXVAL(lignin_struc_below),MINVAL(lignin_struc_below)
	!   WRITE(numout,*) 'DOC',MAXVAL(DOC),MINVAL(DOC)
	!   WRITE(numout,*) 'SOC',MAXVAL(carbon),MINVAL(carbon)
!
!		WRITE(numout,*) 'End of this day'
	ENDIF !IF	NINT(time_counter_ero) .GE. NINT(dt_routing)
    		
    !  
  END SUBROUTINE erosion_main		
    !
    !
!! ================================================================================================================================
!! SUBROUTINE   : erosion_clear
!!
!> BRIEF        : This subroutine deallocates the block memory previously allocated.
!! \n
!_ ================================================================================================================================ 
  SUBROUTINE erosion_clear()
    l_first_erosion = .TRUE.
	!
	IF (ALLOCATED(gridarea))           DEALLOCATE(gridarea)
    IF (ALLOCATED(cveg_f))             DEALLOCATE(cveg_f)
	IF (ALLOCATED(kflitter))           DEALLOCATE(kflitter)
	IF (ALLOCATED(kfroot))             DEALLOCATE(kfroot)
	IF (ALLOCATED(peakrunoff))         DEALLOCATE(peakrunoff)
    IF (ALLOCATED(meanrunoff))         DEALLOCATE(meanrunoff)
	IF (ALLOCATED(peakrundra))         DEALLOCATE(peakrundra)
    IF (ALLOCATED(meanrundra))         DEALLOCATE(meanrundra)	
    IF (ALLOCATED(no_events))          DEALLOCATE(no_events)
	IF (ALLOCATED(eroc_active))        DEALLOCATE(eroc_active)
	IF (ALLOCATED(eroc_slow))          DEALLOCATE(eroc_slow)
	IF (ALLOCATED(eroc_passive))       DEALLOCATE(eroc_passive)
	IF (ALLOCATED(tot_7c))             DEALLOCATE(tot_7c)
	IF (ALLOCATED(tot_7litbelow))      DEALLOCATE(tot_7litbelow)
	IF (ALLOCATED(tot_7doc))           DEALLOCATE(tot_7doc)
	!**************** output variable *************************
  END SUBROUTINE erosion_clear
	!
	!
	!
!! ================================================================================================================================
!! SUBROUTINE   : erosionmap
!!
!> BRIEF        : This subroutine reads and interpolates erosion map
!! \n
!_ ================================================================================================================================ 
  SUBROUTINE erosionmap(nbpt,index,lalo,neighbours,resolution,contfrac,std_totsed,gridarea,effgrid_perc)
    !
    IMPLICIT NONE
	!
	INTEGER(i_std), INTENT(in)                     :: nbpt                  !! Domain size  (unitless)  
    INTEGER(i_std), INTENT(in)                     :: index(nbpt)           !! Index on the global map.
	REAL(r_std), INTENT(in)                        :: lalo(nbpt,2)          !! Vector of latitude and longitudes (beware of the order !)
	INTEGER(i_std), INTENT(in)                     :: neighbours(nbpt,8)    !! Vector of neighbours for each grid point (1=N, 2=E, 3=S, 4=W)
!    INTEGER(i_std), INTENT(out)                    :: maxnbp                !! maxnbp(1):number of fine grid in a ORCHIDEE grid, 
	                                                                        !! 2,3: total row and column number of sub_grid
	REAL(r_std), INTENT(in)                        :: resolution(nbpt,2)    !! The size of each grid box in X and Y (m)	
    REAL(r_std), INTENT(in)                        :: contfrac(nbpt)        !! Fraction of land in each grid box (unitless;0-1)
	CHARACTER(LEN=80)                              :: filename               !! Name of the netcdf file (unitless)
!	INTEGER(i_std)                                 :: nbpmax, nix, njx, fopt !! IDices, (unitless)
	CHARACTER(LEN=30)                              :: callsign               !!
	REAL(r_std), INTENT(out)                       :: std_totsed(nbpt)      !! standard sediment discharge amount rate for each ORCHIDEE cell(ton)
	REAL(r_std), INTENT(out)                       :: effgrid_perc(nbpt)    !! percentage of the effective area in a ORCHIDEE pixel (0-1)
	REAL(r_std), INTENT(out)                       :: gridarea(nbpt)        !! area of each grid cell (m^2)
	!
    INTEGER                                        :: ALLOC_ERR             !!
    LOGICAL                                        :: ok_interpol = .FALSE. !! Flag for interpolation (true/false)
    LOGICAL                                        :: is_root_prc_old		 !!
	INTEGER(i_std)                                 :: iml, jml, lml, tml, fid, ib,ig, jb, ip, jp, itype,subres !! Indices (unitless)
    REAL(r_std)                                    :: lev(1), date, dt, coslat !!
	REAL(r_std)                                    :: lon0,lat0,res_lon,res_lat,cellarea	    !! size (m) of target grid at longitude and latitude direction
	REAL(r_std)                                    :: sumsed				!! Total erosion from one ORCHIDEE cell in the simulation resolution (ton)
	REAL(r_std)                                    :: sumarea				!! total effective area (m^2)
    INTEGER(i_std)                                 :: itau(1)               !!
	
	REAL(r_std), ALLOCATABLE, DIMENSION(:,:)       :: latrel                !! Latitude
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)       :: lonrel                !! Longitude
	REAL(r_std), ALLOCATABLE, DIMENSION(:,:,:)     :: resol_lu              !! Resolution read on the map
	REAL(r_std), ALLOCATABLE, DIMENSION(:,:)       :: totsed          		!! erosion rate for the finer grid, read from forcing data(ton)
	REAL(r_std), ALLOCATABLE, DIMENSION(:,:)       :: effgrid               !! percentage of the effective area in a ORCHIDEE pixel (0-1)
	INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:)    :: mask                  !! Mask to exclude some points (unitless)
	!sub_gridarea, sub_index
!_ ================================================================================================================================	
	!
	is_root_prc_old=is_root_prc
	is_root_prc=.TRUE.
	!
    filename = 'erosion.nc' !
    CALL getin_p('EROSION_FILE',filename)  	
	!
	WRITE(numout,*) 'read info of erosion map', nbpt,filename
    IF (is_root_prc) THEN
       CALL flininfo(filename,iml, jml, lml, tml, fid)
       CALL flinclo(fid)
    ENDIF
    CALL bcast(iml)
    CALL bcast(jml)
    CALL bcast(lml)
    CALL bcast(tml)
	WRITE(numout,*) 'read info of erosion map', nbpt,is_root_prc, filename
    !
	ALLOC_ERR=-1
    ALLOCATE (lonrel(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of lonrel: ",ALLOC_ERR, jml
      STOP 
    ENDIF
	lonrel(:,:)=zero
	!
    ALLOC_ERR=-1
    ALLOCATE (latrel(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of latrel: ",ALLOC_ERR, jml
      STOP 
    ENDIF
	latrel(:,:)=zero
	!
	ALLOC_ERR=-1
    ALLOCATE (totsed(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of totsed: ",ALLOC_ERR, jml
      STOP 
    ENDIF
	totsed(:,:)=zero
	!
	ALLOC_ERR=-1
    ALLOCATE (effgrid(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of effgrid: ",ALLOC_ERR, jml
      STOP 
    ENDIF
	effgrid(:,:)=zero
	!
	ALLOC_ERR=-1
    ALLOCATE(resol_lu(iml,jml,2), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of resol_lu : ",ALLOC_ERR
      STOP 
    ENDIF
	resol_lu(:,:,:)=zero
    !
    ALLOC_ERR=-1
    ALLOCATE(mask(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of mask : ",ALLOC_ERR
      STOP 
    ENDIF
	mask(:,:) = 0.
	!
!	!
	WRITE(numout,*) "Open erosion file",is_root_prc, fid
    IF (is_root_prc) CALL flinopen(filename, .FALSE., iml, jml, lml, lonrel, latrel, lev, tml, itau, date, dt, fid)
	WRITE(numout,*) "Open erosion file1",iml, jml, lml,fid
    CALL bcast(lonrel)
    CALL bcast(latrel)
    CALL bcast(lev)
    CALL bcast(itau)
    CALL bcast(date)
    CALL bcast(dt)
    !
	!WRITE(numout,*) "Open erosion totsed",iml, jml, lml, tml,is_root_prc,fid
    IF (is_root_prc) CALL flinget(fid, 'totsed', iml, jml, lml, tml, 1, 1, totsed)
	!WRITE(numout,*) "Open erosion effgrid_perc",iml, jml, lml, tml,is_root_prc,fid
	IF (is_root_prc) CALL flinget(fid, 'effgrid_perc', iml, jml, lml, tml, 1, 1, effgrid)
	!WRITE(numout,*) "Open erosion fid",iml, jml, lml, tml,is_root_prc,fid
    IF (is_root_prc) CALL flinclo(fid)
	!
	!WRITE(numout,*) 'lat-forcingdata', lalo(1:4,1)
	!WRITE(numout,*) 'lon-forcingdata', lalo(1:4,2)
	WRITE(numout,*) 'lonrel : ', MAXVAL(lonrel), MINVAL(lonrel),lonrel(1:2,1)
    WRITE(numout,*) 'latrel : ', MAXVAL(latrel), MINVAL(latrel),latrel(1,1:2)
!    WRITE(numout,*) 'erosion_rate : ', totsed(:,:)
!	WRITE(numout,*) 'effgrid : ', effgrid(:,:)
	!
	DO ip=1,iml
       DO jp=1,jml
          IF ( totsed(ip,jp).GT. min_sechiba ) THEN
             mask(ip,jp) = 1
          ENDIF
          ! Resolution in longitude
          !
          coslat = MAX( COS( latrel(ip,jp) * pi/180. ), mincos )     
          IF ( ip .EQ. 1 ) THEN
             resol_lu(ip,jp,1) = ABS( lonrel(ip+1,jp) - lonrel(ip,jp) ) * pi/180. * R_Earth * coslat
          ELSEIF ( ip .EQ. iml ) THEN
             resol_lu(ip,jp,1) = ABS( lonrel(ip,jp) - lonrel(ip-1,jp) ) * pi/180. * R_Earth * coslat
          ELSE
             resol_lu(ip,jp,1) = ABS( lonrel(ip+1,jp) - lonrel(ip-1,jp) )/2. * pi/180. * R_Earth * coslat
          ENDIF
          !
          ! Resolution in latitude
          !
          IF ( jp .EQ. 1 ) THEN
             resol_lu(ip,jp,2) = ABS( latrel(ip,jp) - latrel(ip,jp+1) ) * pi/180. * R_Earth
          ELSEIF ( jp .EQ. jml ) THEN
             resol_lu(ip,jp,2) = ABS( latrel(ip,jp-1) - latrel(ip,jp) ) * pi/180. * R_Earth
          ELSE
             resol_lu(ip,jp,2) =  ABS( latrel(ip,jp-1) - latrel(ip,jp+1) )/2. * pi/180. * R_Earth
          ENDIF
          !
       ENDDO
    ENDDO
	!WRITE(numout,*) "lon_resolu=",MAXVAL(resol_lu(:,:,1)), "lat_resolu=",MAXVAL(resol_lu(:,:,2)), iml, jml
	!
	res_lon=lonrel(2,1)-lonrel(1,1)
	res_lat=latrel(1,1)-latrel(1,2)
	lon0=MINVAL(lonrel)-res_lon/2.0
	lat0=MAXVAL(latrel)+res_lat/2.0
	!WRITE(numout,*) 'resolution', res_lon, res_lat
	!WRITE(numout,*) 'Coordinate of WEST-NORTH cornor (row1, col1)', lon0, lat0
	gridarea(:)=zero
	std_totsed(:)=zero
	effgrid_perc(:)=zero
	DO ig =1,nbpt
		ip=floor((lalo(ig,2)-lon0)/res_lon)+1
		jp=floor((lat0-lalo(ig,1))/res_lat)+1
		IF ((ip.GT.zero).AND.(ip.LE.iml).AND.(jp.GT.zero).AND.(jp.LE.jml)) THEN
			gridarea(ig)=resol_lu(ip,jp,1)*resol_lu(ip,jp,2)
			std_totsed(ig)=MAX(totsed(ip,jp),0.0)
			effgrid_perc(ig)=MAX(effgrid(ip,jp),0.0)
		ENDIF !((ip.GT.zero).AND.(ip.LE.iml).AND.(jp.GT.zero).AND.(jp.LE.jml)) THEN
		!WRITE(numout,*) 'Grid_id:', ig,'longitude',lalo(ig,2),'Latitude',lalo(ig,1)
		!WRITE(numout,*) 'Erosion map reading', ip,jp,gridarea(ig),std_totsed(ig),effgrid_perc(ig)
	ENDDO !DO ig =1,nbpt
	is_root_prc=is_root_prc_old
	!   
!   WRITE(numout,*) 'Effective area', gridarea(:) 
!	WRITE(numout,*) 'Effective grid percent', effgrid_perc(:) 
!   WRITE(numout,*) 'standrad sediment loss rate(ton)', std_totsed(:)
	!
	!	 Variable that will be used in erosion_main(**) should not be deallocated   zhc 
	IF (ALLOCATED(latrel))            		DEALLOCATE (latrel)
	IF (ALLOCATED(lonrel))            		DEALLOCATE (lonrel)
	IF (ALLOCATED(mask))            		DEALLOCATE (mask)
    IF (ALLOCATED(resol_lu))            	DEALLOCATE (resol_lu)
	IF (ALLOCATED(totsed))            		DEALLOCATE (totsed)
	IF (ALLOCATED(effgrid))            		DEALLOCATE (effgrid)
    !
  END SUBROUTINE erosionmap 
!
!
!  
END MODULE erosion																																													
																							
